use crate::constants::StaticConstraints;
use crate::controllers::HttpResult;
use crate::extractors::family_extractor::FamilyInPath;
use crate::models::AccommodationID;
use crate::services::{accommodations_list_service, accommodations_reservations_calendars_service};
use actix_web::{web, HttpResponse};

#[derive(serde::Deserialize)]
pub struct CreateCalendarQuery {
    accommodation_id: Option<AccommodationID>,
    name: String,
}

/// Create a calendar
pub async fn create(a: FamilyInPath, req: web::Json<CreateCalendarQuery>) -> HttpResult {
    let accommodation_id = match req.accommodation_id {
        Some(i) => {
            let accommodation = match accommodations_list_service::get_by_id(i).await {
                Ok(a) => a,
                Err(e) => {
                    log::error!("Failed to get accommodation information! {e}");
                    return Ok(HttpResponse::NotFound()
                        .json("The accommodation was not found in the family!"));
                }
            };

            if accommodation.family_id() != a.family_id() {
                return Ok(
                    HttpResponse::NotFound().json("The accommodation was not found in the family!")
                );
            }

            Some(accommodation.id())
        }
        None => None,
    };

    let conf = StaticConstraints::default();
    if !conf.accommodation_calendar_name_len.validate(&req.name) {
        return Ok(HttpResponse::BadRequest().json("Invalid accommodation name!"));
    }

    let calendar = accommodations_reservations_calendars_service::create(
        a.user_id(),
        a.family_id(),
        accommodation_id,
        &req.name,
    )
    .await?;

    Ok(HttpResponse::Ok().json(calendar))
}

/// Get the list of calendars of the user
pub async fn get_list(a: FamilyInPath) -> HttpResult {
    let users =
        accommodations_reservations_calendars_service::get_all_of_user(a.user_id(), a.family_id())
            .await?;
    Ok(HttpResponse::Ok().json(users))
}
